//
//  BlueToothManager.m
//  HandheldBluetooth
//
//  Created by 伟 on 2018/10/23.
//  Copyright © 2018年 ZW. All rights reserved.
//

#import "BlueToothManager.h"
#import "BlueToothManager+SendData.h"
#import "HexHelper.h"
#import <AVFoundation/AVFoundation.h>

static BOOL isRestartConnect = NO;

@interface BlueToothManager ()

@property (nonatomic, strong) BabyBluetooth *baby;

@property (nonatomic, strong) CBCharacteristic * writeCharacteristic;// 写属性

@property (nonatomic, strong) CBCharacteristic * notifyCharacteristic;// 监听属性

@property (nonatomic, assign) BOOL isTagListOrder;   //标签列表搜索命令

@property (nonatomic, strong) NSMutableString *tagListMutStr;

@property (nonatomic, assign) BOOL isLabelOrder;   //写标签命令

@property (nonatomic, strong) NSMutableString *writeLabelMutStr;

@property (nonatomic, assign) BOOL isMemoryOrder;   //内存访问命令

@property (nonatomic, assign) BOOL isMemoryReadOrder;   //内存访问读取命令

@property (nonatomic, strong) AVAudioPlayer *avAudioPlayer;

@property (nonatomic, strong) NSMutableString *readMemoryMutStr;

@end

@implementation BlueToothManager

+ (instancetype)sharedManager {
    static BlueToothManager *bluetoothManager = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        bluetoothManager = [[BlueToothManager alloc] init];
    });
    return bluetoothManager;
}

- (NSMutableArray *)peripheralDataArray {
    if (!_peripheralDataArray) {
        _peripheralDataArray = [NSMutableArray array];
    }
    return _peripheralDataArray;
}
-(BabyBluetooth *)baby{
    if (!_baby) {
        _baby = [BabyBluetooth shareBabyBluetooth];
    }
    return _baby;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        [self initBabyBlueTooth];
        [self connectDelegate];
        [self characteristicsDelegate];
//        self.timeFlag = 0;
    }
    return self;
}

//开始搜索外设
- (void)scanForPeripherals {
    //    //停止之前的连接
    [self.baby cancelAllPeripheralsConnection];
    //    //设置委托后直接可以使用，无需等待CBCentralManagerStatePoweredOn状态。
    self.baby.scanForPeripherals().begin();
}

//停止扫描
-(void)stopScan{
    [self.baby cancelScan];
}

-(void)initBabyBlueTooth{

    [[BabyBluetooth shareBabyBluetooth] setBlockOnCentralManagerDidUpdateState:^(CBCentralManager *central) {
        if (central.state == CBManagerStatePoweredOn) {
            self.bluetoothStateOn = YES;

            //            [self scanForPeripherals];
        }

        if (central.state == CBManagerStatePoweredOff) {
            self.bluetoothStateOn = NO;
            //            if ([self.delegate respondsToSelector:@selector(jwBluetoothManagerDisConnect)]) {
            //                [self.delegate jwBluetoothManagerDisConnect];
            //            }

            NSLog(@"未打开蓝牙");

            if (@available(iOS 11.0, *)) {

                UIAlertController *ui=[UIAlertController alertControllerWithTitle:@"打开蓝牙来允许“HandheldBluetooth”连接到配件" message:@"" preferredStyle:UIAlertControllerStyleAlert];

                UIAlertAction* other=[UIAlertAction actionWithTitle:@"设置" style:UIAlertActionStyleDefault handler:^(UIAlertAction* action){

                    NSURL *url = [NSURL URLWithString:UIApplicationOpenSettingsURLString];// prefs:root=Bluetooth   //App-Prefs:root=Bluetooth
                    if ([[UIApplication sharedApplication] canOpenURL:url])
                    {
                        [[UIApplication sharedApplication] openURL:url];
                    }

                }];
                UIAlertAction* cancel=[UIAlertAction actionWithTitle:@"好" style:UIAlertActionStyleCancel handler:nil

                                       ];

                [ui addAction:cancel];

                [ui addAction:other];

                [[self currentViewController] presentViewController:ui animated:YES completion:nil];
            }

        }
    }];

    //设置发现设service的Characteristics的委托
    [self.baby setBlockOnDiscoverCharacteristics:^(CBPeripheral *peripheral, CBService *service, NSError *error) {
        NSLog(@"===service name:%@",service.UUID);
        for (CBCharacteristic *c in service.characteristics) {
            NSLog(@"charateristic name is :%@",c.UUID);
        }
    }];

    //设置读取characteristics的委托
    [self.baby setBlockOnReadValueForCharacteristic:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
        NSLog(@"1111111characteristic name:%@ value is:%@",characteristics.UUID,characteristics.value);
    }];

    //设置发现characteristics的descriptors的委托
    [self.baby setBlockOnDiscoverDescriptorsForCharacteristic:^(CBPeripheral *peripheral, CBCharacteristic *characteristic, NSError *error) {
        NSLog(@"===characteristic name:%@",characteristic.service.UUID);
        for (CBDescriptor *d in characteristic.descriptors) {
            NSLog(@"CBDescriptor name is :%@",d.UUID);
        }
    }];
    //设置读取Descriptor的委托
    [self.baby setBlockOnReadValueForDescriptors:^(CBPeripheral *peripheral, CBDescriptor *descriptor, NSError *error) {
        NSLog(@"Descriptor name:%@ value is:%@",descriptor.characteristic.UUID, descriptor.value);
    }];

    __weak typeof (self) weakSelf = self;
    [self.baby setBlockOnDiscoverToPeripherals:^(CBCentralManager *central, CBPeripheral *peripheral, NSDictionary *advertisementData, NSNumber *RSSI) {

        //        NSLog(@"peripheral : %@", peripheral.name);
        //        if ([peripheral.name hasPrefix:@"ACI"]) {
        //        }
        //        [weakSelf insertTableView:peripheral advertisementData:advertisementData RSSI:RSSI callBack:^{
        //            // 一直要更新设备信息，所以不能取消扫描，app在使用的时候会一直扫描
        //            //                [[BabyBluetooth shareBabyBluetooth] cancelScan];
        //
        //            if ([weakSelf.delegate respondsToSelector:@selector(jwBluetoothManagerScanedDeviceDictionarys:)]) {
        //                [weakSelf.delegate jwBluetoothManagerScanedDeviceDictionarys:weakSelf.peripheralDataArray];
        //            }
        //        }];

        NSLog(@"搜索到了设备:%@",peripheral.name);
        NSLog(@"搜索到了设备内容:%@",advertisementData[@"kCBAdvDataManufacturerData"]);
        //        NSLog(@"搜索到了设备内容数据字符串:%@",[NSString stringWithFormat:@"%@",advertisementData[@"kCBAdvDataManufacturerData"]]);
        [weakSelf insertTableView:peripheral advertisementData:advertisementData RSSI:RSSI CallBack:^{

            if (weakSelf.delegate && [weakSelf.delegate respondsToSelector:@selector(getAllPeripheralWithArr:)]) {
                [weakSelf.delegate getAllPeripheralWithArr:[weakSelf.peripheralDataArray copy]];
            }

        }];

    }];

    //设置查找设备的过滤器
    [self.baby setFilterOnDiscoverPeripherals:^BOOL(NSString *peripheralName, NSDictionary *advertisementData, NSNumber *RSSI) {
        //最常用的场景是查找某一个前缀开头的设备
        //此项目用广播数据前2个字符串为 "0209" 0x0902 作为标记
        //        NSData * data = [advertisementData valueForKey:@"kCBAdvDataManufacturerData"];
        //        if (data != nil && data.length == 22) {
        //            BOOL isSuccess = [weakSelf resolveAdvertisementData:data];
        //            return isSuccess;
        //        }
        //        return YES;

        //最常用的场景是查找某一个前缀开头的设备
        if ([peripheralName hasPrefix:@"A100"] ) {
            return YES;
        }
        return NO;
    }];


    [self.baby setBlockOnCancelAllPeripheralsConnectionBlock:^(CBCentralManager *centralManager) {
        NSLog(@"setBlockOnCancelAllPeripheralsConnectionBlock");
    }];

    [self.baby setBlockOnCancelScanBlock:^(CBCentralManager *centralManager) {
        NSLog(@"setBlockOnCancelScanBlock");
    }];


    /*设置babyOptions

     参数分别使用在下面这几个地方，若不使用参数则传nil
     - [centralManager scanForPeripheralsWithServices:scanForPeripheralsWithServices options:scanForPeripheralsWithOptions];
     - [centralManager connectPeripheral:peripheral options:connectPeripheralWithOptions];
     - [peripheral discoverServices:discoverWithServices];
     - [peripheral discoverCharacteristics:discoverWithCharacteristics forService:service];

     该方法支持channel版本:
     [baby setBabyOptionsAtChannel:<#(NSString *)#> scanForPeripheralsWithOptions:<#(NSDictionary *)#> connectPeripheralWithOptions:<#(NSDictionary *)#> scanForPeripheralsWithServices:<#(NSArray *)#> discoverWithServices:<#(NSArray *)#> discoverWithCharacteristics:<#(NSArray *)#>]
     */

    //示例:
    //扫描选项->CBCentralManagerScanOptionAllowDuplicatesKey:忽略同一个Peripheral端的多个发现事件被聚合成一个发现事件
    //    NSDictionary *scanForPeripheralsWithOptions = @{CBCentralManagerScanOptionAllowDuplicatesKey:@YES};
    //连接设备->
    //    [baby setBabyOptionsWithScanForPeripheralsWithOptions:scanForPeripheralsWithOptions connectPeripheralWithOptions:nil scanForPeripheralsWithServices:nil discoverWithServices:nil discoverWithCharacteristics:nil];

}

- (void)connectDelegate {
    __weak typeof (self) weakSelf = self;
    //设置设备连接成功的委托,同一个baby对象，使用不同的channel切换委托回调
    [self.baby setBlockOnConnectedAtChannel:channelOnPeropheralView block:^(CBCentralManager *central, CBPeripheral *peripheral) {
        //        [SVProgressHUD showInfoWithStatus:[NSString stringWithFormat:@"设备：%@--连接成功",peripheral.name]];
        weakSelf.currentPeripheral = peripheral;

        NSString *path = [[NSBundle mainBundle] pathForResource:@"connected" ofType:@"mp3"];
        // (2)把音频文件转化成url格式
        NSURL *url = [NSURL fileURLWithPath:path];
        // (3)初始化音频类 并且添加播放文件
        weakSelf.avAudioPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:nil];

        [weakSelf.avAudioPlayer play];

//        AVPlayerItem * songItem = [[AVPlayerItem alloc]initWithURL:url];
//        AVPlayer *avPlayer = [[AVPlayer alloc]initWithPlayerItem:songItem];
//
//        [avPlayer play];


        if (weakSelf.delegate && [weakSelf.delegate respondsToSelector:@selector(connectStateUpdateWithShowStr:AndIsShow:)]) {
            [weakSelf.delegate connectStateUpdateWithShowStr:[NSString stringWithFormat:@"%@：%@%@",Localized(@"设备"),peripheral.name,Localized(@"连接成功")] AndIsShow:YES];
        }

        if (weakSelf.delegate && [weakSelf.delegate respondsToSelector:@selector(connectPeripheralWithState:)]) {
            [weakSelf.delegate connectPeripheralWithState:1];
        }

        isRestartConnect = YES;

    }];

    //设置设备连接失败的委托
    [self.baby setBlockOnFailToConnectAtChannel:channelOnPeropheralView block:^(CBCentralManager *central, CBPeripheral *peripheral, NSError *error) {
        NSLog(@"设备：%@--连接失败",peripheral.name);
        //        [SVProgressHUD showInfoWithStatus:[NSString stringWithFormat:@"设备：%@--连接失败",peripheral.name]];
        weakSelf.currentPeripheral = nil;

        if (weakSelf.delegate && [weakSelf.delegate respondsToSelector:@selector(connectStateUpdateWithShowStr:AndIsShow:)]) {
            [weakSelf.delegate connectStateUpdateWithShowStr:[NSString stringWithFormat:@"%@：%@%@",Localized(@"设备"),peripheral.name,Localized(@"连接失败")] AndIsShow:YES];
        }
    }];

    //设置设备断开连接的委托
    [self.baby setBlockOnDisconnectAtChannel:channelOnPeropheralView block:^(CBCentralManager *central, CBPeripheral *peripheral, NSError *error) {
        NSLog(@"设备：%@--断开连接",peripheral.name);
        //        [SVProgressHUD showInfoWithStatus:[NSString stringWithFormat:@"设备：%@--断开失败",peripheral.name]];
        //        [MBProgressHUD showError:[NSString stringWithFormat:@"设备：%@断开连接失败",peripheral.name] toView:weakSelf.view];
        weakSelf.currentPeripheral = nil;

        NSString *path = [[NSBundle mainBundle] pathForResource:@"disconnected" ofType:@"mp3"];
        // (2)把音频文件转化成url格式
        NSURL *url = [NSURL fileURLWithPath:path];
        // (3)初始化音频类 并且添加播放文件
        weakSelf.avAudioPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:nil];

        [weakSelf.avAudioPlayer play];

//        AVPlayerItem * songItem = [[AVPlayerItem alloc]initWithURL:url];
//        AVPlayer *avPlayer = [[AVPlayer alloc]initWithPlayerItem:songItem];
//
//        [avPlayer play];


        if (weakSelf.delegate && [weakSelf.delegate respondsToSelector:@selector(connectStateUpdateWithShowStr:AndIsShow:)]) {
            [weakSelf.delegate connectStateUpdateWithShowStr:[NSString stringWithFormat:[NSString stringWithFormat:@"%@：%@%@",Localized(@"设备"),peripheral.name,Localized(@"断开连接")],peripheral.name] AndIsShow:YES];
        }

    }];

    //设置发现设备的Services的委托
    [self.baby setBlockOnDiscoverServicesAtChannel:channelOnPeropheralView block:^(CBPeripheral *peripheral, NSError *error) {

    }];
    //设置发现设service的Characteristics的委托
    [self.baby setBlockOnDiscoverCharacteristicsAtChannel:channelOnPeropheralView block:^(CBPeripheral *peripheral, CBService *service, NSError *error) {
        NSLog(@"===service name:%@",service.UUID);
    }];

    //设置读取characteristics的委托
    [self.baby setBlockOnReadValueForCharacteristicAtChannel:channelOnPeropheralView block:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
        NSLog(@"2222222characteristic name:%@ value is:%@",characteristics.UUID,characteristics.value);

        NSLog(@"UUIDString ===== %@",[characteristics.UUID UUIDString]);

//                if ([[characteristics.UUID UUIDString] isEqualToString:@"00001002-0000-1000-8000-00805f9b34fb"]) {
//                    [weakSelf notifyCharacteristics:characteristics];
//                }

                if ([[characteristics.UUID UUIDString] isEqualToString:@"FEE1"]) {
                    
                    [weakSelf notifyCharacteristics:characteristics];
                weakSelf.writeCharacteristic = characteristics;
        //            if ([weakSelf.delegate respondsToSelector:@selector(jwBluetoothManagerDidConnectDevice)]) {
        //                [weakSelf.delegate jwBluetoothManagerDidConnectDevice];
        //            }

                    [[NSNotificationCenter defaultCenter] postNotificationName:@"PeripheralConnectSuccess_Not" object:nil];

                }


    }];
    //设置发现characteristics的descriptors的委托
    [self.baby setBlockOnDiscoverDescriptorsForCharacteristicAtChannel:channelOnPeropheralView block:^(CBPeripheral *peripheral, CBCharacteristic *characteristic, NSError *error) {
        NSLog(@"===characteristic name:%@",characteristic.service.UUID);
        for (CBDescriptor *d in characteristic.descriptors) {
            NSLog(@"CBDescriptor name is :%@",d.UUID);
        }
    }];
    //设置读取Descriptor的委托
    [self.baby setBlockOnReadValueForDescriptorsAtChannel:channelOnPeropheralView block:^(CBPeripheral *peripheral, CBDescriptor *descriptor, NSError *error) {
        NSLog(@"Descriptor name:%@ value is:%@",descriptor.characteristic.UUID, descriptor.value);
    }];

    //读取rssi的委托
    [self.baby setBlockOnDidReadRSSI:^(NSNumber *RSSI, NSError *error) {
        NSLog(@"setBlockOnDidReadRSSI:RSSI:%@",RSSI);
    }];

    //    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(RSSIChange:) name:RSSICHANGE object:nil];

    //扫描选项->CBCentralManagerScanOptionAllowDuplicatesKey:忽略同一个Peripheral端的多个发现事件被聚合成一个发现事件
    NSDictionary *scanForPeripheralsWithOptions = @{CBCentralManagerScanOptionAllowDuplicatesKey:@YES};
    /*连接选项->
     CBConnectPeripheralOptionNotifyOnConnectionKey :当应用挂起时，如果有一个连接成功时，如果我们想要系统为指定的peripheral显示一个提示时，就使用这个key值。
     CBConnectPeripheralOptionNotifyOnDisconnectionKey :当应用挂起时，如果连接断开时，如果我们想要系统为指定的peripheral显示一个断开连接的提示时，就使用这个key值。
     CBConnectPeripheralOptionNotifyOnNotificationKey:
     当应用挂起时，使用该key值表示只要接收到给定peripheral端的通知就显示一个提
     */
    NSDictionary *connectOptions = @{CBConnectPeripheralOptionNotifyOnConnectionKey:@YES,
                                     CBConnectPeripheralOptionNotifyOnDisconnectionKey:@YES,
                                     CBConnectPeripheralOptionNotifyOnNotificationKey:@YES};

    [self.baby setBabyOptionsAtChannel:channelOnPeropheralView scanForPeripheralsWithOptions:scanForPeripheralsWithOptions connectPeripheralWithOptions:connectOptions scanForPeripheralsWithServices:nil discoverWithServices:nil discoverWithCharacteristics:nil];


}

- (void)characteristicsDelegate {
    __weak typeof(self)weakSelf = self;
    //设置读取characteristics的委托
    [self.baby setBlockOnReadValueForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
        NSLog(@"CharacteristicViewController===characteristic name:%@ value is:%@",characteristics.UUID,characteristics.value);
    }];
    //设置发现characteristics的descriptors的委托
    [self.baby setBlockOnDiscoverDescriptorsForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBPeripheral *peripheral, CBCharacteristic *characteristic, NSError *error) {
        NSLog(@"CharacteristicViewController===characteristic name:%@",characteristic.service.UUID);
        for (CBDescriptor *d in characteristic.descriptors) {
            NSLog(@"CharacteristicViewController CBDescriptor name is :%@",d.UUID);
        }
    }];

    //设置读取Descriptor的委托
    [self.baby setBlockOnReadValueForDescriptorsAtChannel:channelOnCharacteristicView block:^(CBPeripheral *peripheral, CBDescriptor *descriptor, NSError *error) {
        NSLog(@"CharacteristicViewController Descriptor name:%@ value is:%@",descriptor.characteristic.UUID, descriptor.value);
    }];

    //设置写数据成功的block
    [self.baby setBlockOnDidWriteValueForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBCharacteristic *characteristic, NSError *error) {
        NSLog(@"setBlockOnDidWriteValueForCharacteristicAtChannel characteristic:%@ and new value:%@",characteristic.UUID, characteristic.value);
    }];

    //设置通知状态改变的block
    [self.baby setBlockOnDidUpdateNotificationStateForCharacteristicAtChannel:channelOnCharacteristicView block:^(CBCharacteristic *characteristic, NSError *error) {
        NSLog(@"uid:%@,isNotifying:%@",characteristic.UUID,characteristic.isNotifying?@"on":@"off");
    }];
}

-(void)readInfoData{
    self.baby.channel(channelOnCharacteristicView).characteristicDetails(self.currentPeripheral,self.writeCharacteristic);
}

#pragma mark -- 扫描获取设备
//插入table数据
-(void)insertTableView:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI CallBack:(void(^)(void))callBlock{

    NSArray *peripherals = [self.peripheralDataArray valueForKey:@"peripheral"];
    if(![peripherals containsObject:peripheral]) {
        // 未保存
        NSMutableDictionary *item = [[NSMutableDictionary alloc] init];
        [item setValue:peripheral forKey:@"peripheral"];
        [item setValue:RSSI forKey:@"RSSI"];
        [item setValue:advertisementData forKey:@"advertisementData"];
        //        JWDeviceModel * model = [self resolveData:advertisementData[@"kCBAdvDataManufacturerData"]];
        //        model.kCBAdvDataLocalName = advertisementData[@"kCBAdvDataLocalName"];
        //        model.RSSI = RSSI;
        //        [item setObject:model forKey:@"JWDeviceModel"];
        [self.peripheralDataArray addObject:item];
    }else {
        // 已保存
        NSUInteger index = [peripherals indexOfObject:peripheral];
        NSMutableDictionary *item = [[NSMutableDictionary alloc] init];
        [item setValue:peripheral forKey:@"peripheral"];
        [item setValue:RSSI forKey:@"RSSI"];
        [item setValue:advertisementData forKey:@"advertisementData"];

        //        JWDeviceModel * model = [self resolveData:advertisementData[@"kCBAdvDataManufacturerData"]];
        //        model.kCBAdvDataLocalName = advertisementData[@"kCBAdvDataLocalName"];
        //        model.RSSI = RSSI;
        //        [item setObject:model forKey:@"JWDeviceModel"];
        [self.peripheralDataArray replaceObjectAtIndex:index withObject:item];
    }

    if (callBlock) {
        callBlock();
    }
}

#pragma mark -- 连接设备
- (void)connectPeripheral:(CBPeripheral *)peripheral {
    if (!peripheral || peripheral.state == CBPeripheralStateConnecting) {
        return;
    }

    [self.baby cancelScan];
    self.baby.having(peripheral).and.channel(channelOnPeropheralView).then.connectToPeripherals().discoverServices().discoverCharacteristics().readValueForCharacteristic().discoverDescriptorsForCharacteristic().readValueForDescriptors().begin();

}

- (void)disconnectPeripheral:(CBPeripheral *)peripheral {
    if (!peripheral || peripheral.state == CBPeripheralStateConnecting) {
        return;
    }
    //    self.currentDevice = nil;
    self.currentPeripheral = nil;
    self.writeCharacteristic = nil;

    //注意断开连接必须取消监听！！！否则下次不清空再次连接上不能监听到信息
        if (self.notifyCharacteristic.isNotifying) {
            // 取消监听
            [self.baby cancelNotify:self.currentPeripheral characteristic:self.notifyCharacteristic];
        }

    [self.baby cancelPeripheralConnection:peripheral];
}


#pragma mark - Order

/**
 写命令

 @param string 十六进制字符串
 */
- (void)wirteData:(NSString *)string {
    if(self.currentPeripheral.state != CBPeripheralStateConnected || !self.writeCharacteristic) {
        //        [SVProgressHUD showErrorWithStatus:@"peripheral已经断开连接，请重新连接"];
        return;
    }

    self.isTagListOrder = NO;
    self.isLabelOrder = NO;
    self.isMemoryOrder = NO;
    self.isMemoryReadOrder = NO;

    [self.currentPeripheral writeValue:[self hexToBytes:string] forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithResponse];
}

/**
 监听

 @param characteristics 特征
 */
- (void)notifyCharacteristics:(CBCharacteristic *)characteristics {
    if(self.currentPeripheral.state != CBPeripheralStateConnected) {
        //        [SVProgressHUD showErrorWithStatus:@"peripheral已经断开连接，请重新连接"];
        return;
    }

    self.notifyCharacteristic = characteristics;

    __weak typeof (self) weakSelf = self;
    [weakSelf.currentPeripheral setNotifyValue:YES forCharacteristic:characteristics];
    [weakSelf.baby notify:weakSelf.currentPeripheral
           characteristic:characteristics
                    block:^(CBPeripheral *peripheral, CBCharacteristic *characteristics, NSError *error) {
                        NSLog(@"notifyCharacteristics === %@",characteristics);
//                        NSLog(@"stringFromHexString === %@",[HexHelper stringFromHexString:characteristics.value]);
//                        NSLog(@"convertHexStrToString ==== %@",[HexHelper convertHexStrToString:characteristics.value]);
//                        NSLog(@"getBinaryByhex === %@",[HexHelper getBinaryByhex:characteristics.value]);
                        NSLog(@"toString ==== %@",[HexHelper toString:characteristics.value]);
//                        NSString * flagString = [[HexHelper toString:characteristics.value] substringWithRange:NSMakeRange(0*2, 2)];
//                        NSLog(@"flagString ==== %@",flagString);

                        NSLog(@"toASCIIString ==== %@",[HexHelper toASCIIString:[HexHelper toString:characteristics.value]]);

                        [weakSelf resolveByNotifyData:characteristics.value];
                    }];
}

/**
 数据处理

 @param data 原始数据
 */
- (void)resolveByNotifyData:(NSData *)data {

    NSString *dataString = [HexHelper toString:data];

    if (dataString.length == 0) {
        return;
    }

    if (self.isMemoryOrder) {

        if (dataString.length > 8) {

            //内存访问反馈
            NSString * hexTagListString = [dataString substringWithRange:NSMakeRange(0, 4)];

            if ([hexTagListString isEqualToString:@"3E45"]) {

                NSString * formalString = [dataString substringWithRange:NSMakeRange(4, dataString.length - 4 - 4)];
                NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

                if (_memoryDelegate && [_memoryDelegate respondsToSelector:@selector(getMemoryAccessWithReplyStr:)]) {
                    [_memoryDelegate getMemoryAccessWithReplyStr:ASCIIStr];
                }

            }

        }

    }else if (self.isMemoryReadOrder){

        self.readMemoryMutStr = [[self.readMemoryMutStr stringByAppendingString:[HexHelper toString:data]] mutableCopy];

        if (_memoryDelegate && [_memoryDelegate respondsToSelector:@selector(getMemoryReadWithReadStr:)]) {
            [_memoryDelegate getMemoryReadWithReadStr:[self.readMemoryMutStr copy]];
        }


        if (dataString.length > 4) {            

            //内存访问反馈
            NSString * hexTagListString = [dataString substringWithRange:NSMakeRange(0, 4)];

            if ([hexTagListString isEqualToString:@"3E45"]) {

                NSString * formalString = [dataString substringWithRange:NSMakeRange(4, dataString.length - 4 - 4)];
                NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

                if (_memoryDelegate && [_memoryDelegate respondsToSelector:@selector(getMemoryAccessWithReplyStr:)]) {
                    [_memoryDelegate getMemoryAccessWithReplyStr:ASCIIStr];
                }

            }


        }

    } else if (self.isLabelOrder) {

        self.writeLabelMutStr = [[self.writeLabelMutStr stringByAppendingString:[HexHelper toString:data]] mutableCopy];

        if (_writeDelegate && [_writeDelegate respondsToSelector:@selector(getWriteLabelWithReplyStr:)]) {
            [_writeDelegate getWriteLabelWithReplyStr:[self.writeLabelMutStr copy]];
        }

    }else if (self.isTagListOrder) {

//        NSLog(@"self.tagListMutStr === %@",self.tagListMutStr);

        __weak typeof(self) weakSelf = self;

        if (dataString.length > 12) {

            //清空搜索列表通知
            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 12)];

            //>f 1 3
            if ([hexWritePowerString isEqualToString:@"3E6620312033"]) {

                //                NSString * formalString = [dataString substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
                //                NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

                self.tagListMutStr = [NSMutableString string];

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(clearSearchListNot)]) {
                    [_tagListDelegate clearSearchListNot];
                }

            }

        }

        if (dataString.length > 6) {

            //重启RFID搜索
            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 6)];

            if ([hexWritePowerString isEqualToString:@"3E6E20"] || [hexWritePowerString isEqualToString:@"3E4E20"]) {

                NSString * formalString = [dataString substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
                NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

                if (weakSelf.tagListDelegate && [weakSelf.tagListDelegate respondsToSelector:@selector(getTagListTotal:)]) {
                    [weakSelf.tagListDelegate getTagListTotal:ASCIIStr];
                }

            }

            if ([hexWritePowerString isEqualToString:@"3E4133"]) {

                if (weakSelf.tagListDelegate && [weakSelf.tagListDelegate respondsToSelector:@selector(getStopOrder)]) {
                    [weakSelf.tagListDelegate getStopOrder];
                }

            }

            if ([hexWritePowerString isEqualToString:@"3E4166"]) {

//                dispatch_async(dispatch_get_main_queue(), ^{

//                    self.tagListMutStr = [NSMutableString string];

                    self.isTagListOrder = YES;

                    if (weakSelf.tagListDelegate && [weakSelf.tagListDelegate respondsToSelector:@selector(restartRFIDSearch)]) {
                        [weakSelf.tagListDelegate restartRFIDSearch];
                    }
//                });

            }

        }


        if (!isRestartConnect) {

        if (dataString.length > 8) {

            //扫码
            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 8)];

            if ([hexWritePowerString isEqualToString:@"3E362032"]) {

                self.isTagListOrder = NO;

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(ScavengingState:)]) {
                    [_tagListDelegate ScavengingState:YES];
                }

            }

            //RFID扫描
            NSString * RFIDScanString = [dataString substringWithRange:NSMakeRange(0, 8)];

            if ([RFIDScanString isEqualToString:@"3E362030"]) {

                self.isTagListOrder = YES;

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(ScavengingState:)]) {
                    [_tagListDelegate ScavengingState:NO];
                }

            }
        }

            isRestartConnect = NO;
        }


        if (dataString.length > 4) {

        //通配符搜索数据
        NSString * hexTagListWildcardString = [dataString substringWithRange:NSMakeRange(0, 4)];

        if ([hexTagListWildcardString isEqualToString:@"3E47"]) {

            NSString * formalString = [dataString substringWithRange:NSMakeRange(4, dataString.length - 4 - 4)];
            NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

            NSArray *wailcardArr = [ASCIIStr componentsSeparatedByString:@","];
            ASCIIStr = [wailcardArr lastObject];

            if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getWailcardTagListWithStr:)]) {
                [_tagListDelegate getWailcardTagListWithStr:ASCIIStr];
            }

        }else{
            if (self.tagListMutStr.length <= 0) {
                self.tagListMutStr = [[HexHelper toString:data] mutableCopy];
            }else{
            self.tagListMutStr = [[self.tagListMutStr stringByAppendingString:[HexHelper toString:data]] mutableCopy];
            }

            if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getTagListWithStr:)]) {
                [_tagListDelegate getTagListWithStr:[self.tagListMutStr copy]];
            }
        }
        }else{

            if (self.tagListMutStr.length <= 0) {
                self.tagListMutStr = [[HexHelper toString:data] mutableCopy];
            }else{
            self.tagListMutStr = [[self.tagListMutStr stringByAppendingString:[HexHelper toString:data]] mutableCopy];
            }

            if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getTagListWithStr:)]) {
                [_tagListDelegate getTagListWithStr:[self.tagListMutStr copy]];
            }

        }

//        if (dataString.length > 4) {
//
//        //扫码
//        NSString * sweepYardString = [dataString substringWithRange:NSMakeRange(0, 4)];
//
//        if ([sweepYardString isEqualToString:@"3E4A"]) {
//
//            NSString * formalString = [dataString substringWithRange:NSMakeRange(4, dataString.length - 4 - 4)];
//            NSString * ASCIIStr = [HexHelper toASCIIString:formalString];
//
//            if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getSweepYardWithStr:)]) {
//                [_tagListDelegate getSweepYardWithStr:ASCIIStr];
//            }
//
//        }
//        }


//        if (dataString.length > 6) {
//            //重启 RFID搜索结尾
//            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 6)];
//
//            if ([hexWritePowerString isEqualToString:@"3E4133"]) {
//
//                //                self.tagListMutStr = [NSMutableString string];
//                //
//                //                self.isTagListOrder = NO;
//
//
//            }
//        }


    }else{

        NSString * hexString = [HexHelper toString:data];

        if ([[HexHelper toString:data] length] > 6) {
            hexString = [[HexHelper toString:data] substringWithRange:NSMakeRange(0, 6)];
        }

        if (dataString.length > 12) {

            //清空搜索列表通知
            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 12)];

            //>f 1 3
            if ([hexWritePowerString isEqualToString:@"3E6620312033"]) {

                //                NSString * formalString = [dataString substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
                //                NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

                self.tagListMutStr = [NSMutableString string];

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(clearSearchListNot)]) {
                    [_tagListDelegate clearSearchListNot];
                }

            }

        }

        if (dataString.length > 6) {

            //重启RFID搜索
            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 6)];

            if ([hexWritePowerString isEqualToString:@"3E4166"]) {

                if (!self.tagListMutStr) {

                self.tagListMutStr = [NSMutableString string];
                }

                self.isTagListOrder = YES;

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(restartRFIDSearch)]) {
                    [_tagListDelegate restartRFIDSearch];
                }

            }

        }

    //电池电量
    if ([hexString isEqualToString:@"3E7620"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralElectricityWithStr:)]) {
            [_dataDelegate getPeripheralElectricityWithStr:electricityStr];
        }
    }
    //RFID功率大小
    if ([hexString isEqualToString:@"3E7020"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralRFIDPowerWithStr:)]) {
            [_dataDelegate getPeripheralRFIDPowerWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E6520"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralReadSpeedWithStr:)]) {
            [_dataDelegate getPeripheralReadSpeedWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E6B20"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralCriticalWithStr:)]) {
            [_dataDelegate getPeripheralCriticalWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E7520"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralReduceUnitWithStr:)]) {
            [_dataDelegate getPeripheralReduceUnitWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E7320"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralVolumeWithStr:)]) {
            [_dataDelegate getPeripheralVolumeWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E6220"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralNewLableWithStr:)]) {
            [_dataDelegate getPeripheralNewLableWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E6320"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralContinueReadWithStr:)]) {
            [_dataDelegate getPeripheralContinueReadWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E6120"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralAllowMeauWithStr:)]) {
            [_dataDelegate getPeripheralAllowMeauWithStr:electricityStr];
        }
    }

    if ([hexString isEqualToString:@"3E3120"]) {
        NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
        NSString *electricityStr = [HexHelper toASCIIString:formalString];
        if (_dataDelegate && [_dataDelegate respondsToSelector:@selector(getPeripheralLanguageWithStr:)]) {
            [_dataDelegate getPeripheralLanguageWithStr:electricityStr];
        }
    }

        //读取类型
        if ([hexString isEqualToString:@"3E3620"]) {
            NSString * formalString = [[HexHelper toString:data] substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
            NSString *typeStr = [HexHelper toASCIIString:formalString];
            if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getReadType:)]) {
                [_tagListDelegate getReadType:typeStr];
            }
        }

        if (dataString.length > 4) {

        //标签列表数据
        NSString * hexTagListString = [dataString substringWithRange:NSMakeRange(0, 4)];

        if ([hexTagListString isEqualToString:@"3E54"]) {

            NSString * formalString = [dataString substringWithRange:NSMakeRange(4, dataString.length - 4 - 4)];
            NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

            if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getClearTagListWithStr:)]) {
                [_tagListDelegate getClearTagListWithStr:ASCIIStr];
            }

        }

        //通配符搜索数据
        NSString * hexTagListWildcardString = [dataString substringWithRange:NSMakeRange(0, 4)];

        if ([hexTagListWildcardString isEqualToString:@"3E47"]) {

            NSString * formalString = [dataString substringWithRange:NSMakeRange(4, dataString.length - 4 - 4)];
            NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

            NSArray *wailcardArr = [ASCIIStr componentsSeparatedByString:@","];
            ASCIIStr = [wailcardArr lastObject];

            if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getWailcardTagListWithStr:)]) {
                [_tagListDelegate getWailcardTagListWithStr:ASCIIStr];
            }

        }

        //写功率数据
        NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 4)];

            //3E35
        if ([hexWritePowerString isEqualToString:@"3E70"]) {

            NSString * formalString = [dataString substringWithRange:NSMakeRange(6, dataString.length - 6 - 4)];
            NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

            if (_writeDelegate && [_writeDelegate respondsToSelector:@selector(getWritePowerWithStr:)]) {
                [_writeDelegate getWritePowerWithStr:ASCIIStr];
            }

        }


            //扫码
            NSString * sweepYardString = [dataString substringWithRange:NSMakeRange(0, 4)];

            if ([sweepYardString isEqualToString:@"3E4A"]) {

                NSString * formalString = [dataString substringWithRange:NSMakeRange(4, dataString.length - 4 - 4)];
                NSString * ASCIIStr = [HexHelper toASCIIString:formalString];

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(getSweepYardWithStr:)]) {
                    [_tagListDelegate getSweepYardWithStr:ASCIIStr];
                }

            }

        }



//        if (dataString.length > 6) {
//            //重启 RFID搜索结尾
//            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 6)];
//
//            if ([hexWritePowerString isEqualToString:@"3E4133"]) {
//
//                self.tagListMutStr = [NSMutableString string];
//
//                self.isTagListOrder = NO;
//            }
//        }

        if (dataString.length > 8) {

            //扫码
            NSString * hexWritePowerString = [dataString substringWithRange:NSMakeRange(0, 8)];

            if ([hexWritePowerString isEqualToString:@"3E362032"]) {

                self.isTagListOrder = NO;

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(ScavengingState:)]) {
                    [_tagListDelegate ScavengingState:YES];
                }

            }

            //RFID扫描
            NSString * RFIDScanString = [dataString substringWithRange:NSMakeRange(0, 8)];

            if ([RFIDScanString isEqualToString:@"3E362030"]) {

                self.isTagListOrder = YES;

                if (_tagListDelegate && [_tagListDelegate respondsToSelector:@selector(ScavengingState:)]) {
                    [_tagListDelegate ScavengingState:NO];
                }

            }
        }

    }
}

#pragma mark - Special Order
- (void)wirteTagListSearchData:(NSString *)string {
    if(self.currentPeripheral.state != CBPeripheralStateConnected || !self.writeCharacteristic) {
        //        [SVProgressHUD showErrorWithStatus:@"peripheral已经断开连接，请重新连接"];
        return;
    }

    self.isTagListOrder = YES;
    self.isLabelOrder = NO;
    self.isMemoryOrder = NO;
    self.isMemoryReadOrder = NO;

    self.tagListMutStr = [NSMutableString string];

    [self.currentPeripheral writeValue:[self hexToBytes:string] forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithResponse];
}
- (void)wirteTagListContinueSearchData:(NSString *)string {
    if(self.currentPeripheral.state != CBPeripheralStateConnected || !self.writeCharacteristic) {
        //        [SVProgressHUD showErrorWithStatus:@"peripheral已经断开连接，请重新连接"];
        return;
    }

    self.isTagListOrder = YES;
    self.isLabelOrder = NO;
    self.isMemoryOrder = NO;
    self.isMemoryReadOrder = NO;

    [self.currentPeripheral writeValue:[self hexToBytes:string] forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithResponse];
}

- (void)wirteLabelData:(NSString *)string {

    if(self.currentPeripheral.state != CBPeripheralStateConnected || !self.writeCharacteristic) {
        //        [SVProgressHUD showErrorWithStatus:@"peripheral已经断开连接，请重新连接"];
        return;
    }

    self.isTagListOrder = NO;
    self.isLabelOrder = YES;
    self.isMemoryOrder = NO;
    self.isMemoryReadOrder = NO;

    self.writeLabelMutStr = [NSMutableString string];

    [self.currentPeripheral writeValue:[self hexToBytes:string] forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithResponse];

}
-(void)wirteMemoryAccessData:(NSString *)string{

    if(self.currentPeripheral.state != CBPeripheralStateConnected || !self.writeCharacteristic) {
        //        [SVProgressHUD showErrorWithStatus:@"peripheral已经断开连接，请重新连接"];
        return;
    }

    self.isTagListOrder = NO;
    self.isLabelOrder = NO;
    self.isMemoryOrder = YES;
    self.isMemoryReadOrder = NO;

    [self.currentPeripheral writeValue:[self hexToBytes:string] forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithResponse];

}

-(void)wirteMemoryAccessReadData:(NSString *)string{

    if(self.currentPeripheral.state != CBPeripheralStateConnected || !self.writeCharacteristic) {
        //        [SVProgressHUD showErrorWithStatus:@"peripheral已经断开连接，请重新连接"];
        return;
    }

    self.isTagListOrder = NO;
    self.isLabelOrder = NO;
    self.isMemoryOrder = NO;
    self.isMemoryReadOrder = YES;

    self.readMemoryMutStr = [NSMutableString string];

    [self.currentPeripheral writeValue:[self hexToBytes:string] forCharacteristic:self.writeCharacteristic type:CBCharacteristicWriteWithResponse];

}

#pragma mark -- Other

-(UIViewController *)currentViewController
{

    UIViewController * currVC = nil;
    UIWindow *keyWin = [UIApplication sharedApplication].keyWindow;
    UIViewController * Rootvc = keyWin.rootViewController;
    do {
        if ([Rootvc isKindOfClass:[UINavigationController class]]) {
            UINavigationController * nav = (UINavigationController *)Rootvc;
            UIViewController * v = [nav.viewControllers lastObject];
            currVC = v;
            Rootvc = v.presentedViewController;
            continue;
        }else if([Rootvc isKindOfClass:[UITabBarController class]]){
            UITabBarController * tabVC = (UITabBarController *)Rootvc;
            currVC = tabVC;
            Rootvc = [tabVC.viewControllers objectAtIndex:tabVC.selectedIndex];
            continue;
        }
    } while (Rootvc!=nil);

    return currVC;
}

@end
